package ecp.vcs.com.ecpsyncplugin.re_sync.sync;

import android.content.Context;
import android.net.Uri;
import android.util.Log;

import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.text.DecimalFormat;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import ecp.vcs.com.ecpsyncplugin.re_sync.SynchronizationService;
import ecp.vcs.com.ecpsyncplugin.re_sync.database.DataBaseHelper;
import ecp.vcs.com.ecpsyncplugin.re_sync.database.DatabaseHelperExtended;
import ecp.vcs.com.ecpsyncplugin.re_sync.interfaces.SyncUploadCallback;
import ecp.vcs.com.ecpsyncplugin.re_sync.logging.LogHelper;
import ecp.vcs.com.ecpsyncplugin.re_sync.networking.AppUtil;
import ecp.vcs.com.ecpsyncplugin.re_sync.utils.Const;
import ecp.vcs.com.ecpsyncplugin.re_sync.utils.Content;
import ecp.vcs.com.ecpsyncplugin.re_sync.utils.SyncErrorCode;


public class SyncUploadHelper {

    private final Context context;
    private final DataBaseHelper dataBaseHelper;
    private final DatabaseHelperExtended databaseHelperExtended;
    private final SyncUploadCallback callback;
    private String exceptionDetails = "";
    private String syncLogId = "";
    private String uploadFileSize = "";
    private boolean byPassSSL=true;


    public SyncUploadHelper(Context context, SyncUploadCallback callback) {
        this.context = context;
        dataBaseHelper = new DataBaseHelper(context);
        databaseHelperExtended = new DatabaseHelperExtended(context);
        this.callback = callback;
    }


    /**
     * Get File Size
     *
     * @param sourceFile
     * @return
     */
    private String getFileSize(File sourceFile) {
        String output = "";
        try {
            int length = (int) sourceFile.length();
            double kilobytes = ((double) length / 1024);
            DecimalFormat df = new DecimalFormat("#.##");
            output = df.format(kilobytes);
        } catch (Exception e) {
            exceptionDetails = AppUtil.getStackTrace(e);
            e.printStackTrace();
        }
        return output;
    }


    /**
     * Upload File With File Type
     *
     * @param filePath
     * @param uploadUrl
     * @param uploadType Upload Type- T For Transaction
     *                   G For Get Data
     */
    public void uploadFileWithType(String filePath, String uploadUrl, String uploadType,int startProgress,int endProgress,String syncType,String token,String imeiNumber,String userID) {

        String error_code = "";
        exceptionDetails = "";
        long FUpload_Start = System.currentTimeMillis();
        HttpsURLConnection conn;
        DataOutputStream dos;
        String lineEnd = "\r\n";
        String twoHyphens = "--";
        String boundary = "------fileUpload";
        int bytesRead, bytesAvailable, bufferSize;
        byte[] buffer;
        int maxBufferSize = 1 * 1024 * 1024;
        File sourceFile = new File(filePath);

        callback.onProgressUpdate(startProgress+10);

        try {

            if (byPassSSL){
                SSLContext ctx = SSLContext.getInstance("TLS");
                ctx.init(null, new TrustManager[] {
                        new X509TrustManager() {
                            public void checkClientTrusted(X509Certificate[] chain, String authType) {}
                            public void checkServerTrusted(X509Certificate[] chain, String authType) {}
                            public X509Certificate[] getAcceptedIssuers() { return new X509Certificate[]{}; }
                        }
                }, null);
                HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

                HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
                    public boolean verify(String hostname, SSLSession session) {
                        return true;
                    }
                });
            }

            FileInputStream fileInputStream = new FileInputStream(sourceFile);
            Log.v("FileUploadUrl", uploadUrl);
            URL url = new URL(uploadUrl);
            Log.v("FileUploadUrl", url.toString());
            // Open a HTTP  connection to  the URL
            callback.onProgressUpdate(startProgress+15);
            conn = (HttpsURLConnection) url.openConnection();
            conn.setConnectTimeout(1800000);
            conn.setChunkedStreamingMode(0);
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use a Cached Copy
            conn.setRequestMethod("POST");
            callback.onProgressUpdate(startProgress+20);
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("ENCTYPE", "multipart/form-data");
            conn.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + boundary);
            callback.onProgressUpdate(startProgress+21);
            if (syncType != null && !syncType.equalsIgnoreCase("F")) {
                conn.setRequestProperty("filename", "");
            } else {
                conn.setRequestProperty("filename", filePath);
            }
            callback.onProgressUpdate(startProgress+22);
            conn.setRequestProperty("token", token);
            conn.setRequestProperty("user_id", userID);
            conn.setRequestProperty("upload_type", uploadType);
            Log.e("Upload Zip","====token==="+token);
            Log.e("Upload Zip","====filePath==="+filePath);
            Log.e("Upload Zip","====uploadType==="+uploadType);
            String lastDate = databaseHelperExtended.getSyncDateAndTime();
            conn.setRequestProperty("first_sync", !lastDate.isEmpty()? "N":"Y");
            LogHelper.printLog("TAG_UPLOAD_FIRST","ISFIRST_SYNC::: "+(!lastDate.isEmpty()? "N":"Y"));
//            if (uploadType.equalsIgnoreCase("G")) {
//                String lastDate = databaseHelperExtended.getSyncDateAndTime();
//                conn.setRequestProperty("first_sync", !lastDate.isEmpty()? "N":"Y");
//                LogHelper.printLog("TAG_UPLOAD_FIRST","ISFIRST_SYNC::: "+(!lastDate.isEmpty()? "N":"Y"));
//            } else {
//                String lastDate = databaseHelperExtended.getTransactionSyncDateAndTime();
//                conn.setRequestProperty("first_sync", !lastDate.isEmpty()? "N":"Y");
//                LogHelper.printLog("TAG_UPLOAD_FIRST","ISFIRST_SYNC::: "+(!lastDate.isEmpty()? "N":"Y"));
//            }

            conn.setRequestProperty(Const.WEB_SERVICES_PARAM.KEY_IMEI, imeiNumber);

            Log.e("Upload Zip","====conn==="+conn);

            callback.onProgressUpdate(startProgress+25);

            dos = new DataOutputStream(conn.getOutputStream());
            callback.onProgressUpdate(startProgress+26);
            uploadFileSize = getFileSize(sourceFile);
            callback.onProgressUpdate(startProgress+30);
            dos.writeBytes(twoHyphens + boundary + lineEnd);
            dos.writeBytes("Content-Disposition: form-data; name=\"file\";filename=\"" + filePath + "\"" + lineEnd);
            dos.writeBytes(lineEnd);
            callback.onProgressUpdate(startProgress+35);
            // create a buffer of  maximum size
            bytesAvailable = fileInputStream.available();
            bufferSize = Math.min(bytesAvailable, maxBufferSize);
            buffer = new byte[bufferSize];
            callback.onProgressUpdate(startProgress+36);
            // read file and write it into form...
            bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            callback.onProgressUpdate(startProgress+37);
            while (bytesRead > 0) {
                dos.write(buffer, 0, bufferSize);
                bytesAvailable = fileInputStream.available();
                bufferSize = Math.min(bytesAvailable, maxBufferSize);
                bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            }
            callback.onProgressUpdate(startProgress+40);
            // send multipart form data necesssary after file data...
            dos.writeBytes(lineEnd);
            callback.onProgressUpdate(startProgress+41);
            dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);
            callback.onProgressUpdate(startProgress+43);
            // Responses from the server (code and message)
            int serverResponseCode = conn.getResponseCode();
            callback.onProgressUpdate(startProgress+45);
            String serverResponseMessage = conn.getResponseMessage();
            Log.i("RESPONSE_TAG", "HTTP Response is : " + serverResponseMessage + ": " + serverResponseCode);

            // ------------------ read the SERVER RESPONSE
//            DataInputStream inStream;
            try {
                BufferedReader inStream
                        = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                DataInputStream x = new DataInputStream(conn.getInputStream());
                String str="";
//                String strTest=readFullyAsString(x,"UTF-8");
                callback.onProgressUpdate(startProgress+45);

                while ((str = inStream.readLine()) != null) {
                    if (!str.trim().isEmpty()){
                        Log.e("FileUploadSOFRESPONSE", "SOF Server Response" + str);
                        LogHelper.printLog("TIME_forSync", "FileUploadStopTimer" + System.currentTimeMillis());
                        if(!str.trim().isEmpty()){
                            JSONObject jsonObject = new JSONObject(str);
                            String message = jsonObject.getString("message");
                            if (jsonObject.optBoolean("status")) {

                                String date_folder = jsonObject.getString("date_folder");
                                syncLogId = jsonObject.getString("sync_log_id");
                                LogHelper.printLog("SYNC_LOG_ID_SYNC", "SYNC LOG ID::: " + syncLogId);
//                                Content.setPreferenceValue(context, Const.dateUpload, date_folder);
                                SynchronizationService.dateFolder = date_folder;
                                serverResponseCode = 200;

                            } else {

                                serverResponseCode = 100;
                                error_code = jsonObject.optString("msg_code");
                                exceptionDetails = message;

                            }
                        }
                    }
                }
                inStream.close();
                callback.onProgressUpdate(startProgress+45);
            } catch (IOException ioex) {
                exceptionDetails = AppUtil.getStackTrace(ioex);
                callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
                databaseHelperExtended.insertTransactionErrorLog(context, this.getClass().getSimpleName(), "uploadFile", exceptionDetails, exceptionDetails, "",syncLogId,userID);
                Log.e("FileUploadSOFERROR", "SOF error: " + ioex.getMessage(), ioex);
            }

            fileInputStream.close();
            dos.flush();
            dos.close();

//            Check if Response is SuccessFull Or Nor
            if (serverResponseCode == 200) {
                callback.onProgressUpdate(startProgress+45);
                long FUpload_End = System.currentTimeMillis();
                Log.e("TAG_SYNC_FILE_UPLOAD", "SYNC FILE UPLOADED SUCCESSFULLY");
                try {
                    long difference = AppUtil.getMillisDifference(FUpload_Start, FUpload_End);
                    Log.v("SYNCTRACK", "UFS_ TIME::" + difference);

                    Uri uri = Uri.fromFile(new File(filePath));
                    callback.onProgressUpdate(startProgress+45);
                    callback.onFileUploadSuccess(uri.getLastPathSegment(), syncLogId, String.valueOf(difference), uploadFileSize,uploadType);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {

                Log.e("TAG_SYNC_FILE_UPLOAD", "SYNC FILE UPLOADED FAILED");
                if (error_code.equalsIgnoreCase(SyncErrorCode.ERROR_501)) {
                    callback.onFileUploadFail(Integer.parseInt(SyncErrorCode.ERROR_501), exceptionDetails, uploadFileSize,uploadType);
                } else {
                    exceptionDetails = exceptionDetails + " File Upload Server Error:" + serverResponseCode;
                    callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
                }

            }
        } catch (MalformedURLException ex) {
            exceptionDetails = AppUtil.getStackTrace(ex);
            callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
            ex.printStackTrace();
        } catch (Exception e) {
            exceptionDetails = AppUtil.getStackTrace(e);
            callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
            e.printStackTrace();
        }
    }

    public String readFullyAsString(InputStream inputStream, String encoding) throws IOException {
        return readFully(inputStream).toString(encoding);
    }

    private ByteArrayOutputStream readFully(InputStream inputStream) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int length = 0;
        while ((length = inputStream.read(buffer)) != -1) {
            baos.write(buffer, 0, length);
        }
        return baos;
    }

    /**
     * Upload File With File Type
     *
     * @param filePath
     * @param uploadUrl
     * @param uploadType Upload Type- T For Transaction
     *                   G For Get Data
     */
    public void uploadFileWithTypeAutoSync(String filePath, String uploadUrl, String uploadType,String syncType,String token,String imeiNumber,String userID) {

        String error_code = "";
        exceptionDetails = "";
        long FUpload_Start = System.currentTimeMillis();
        HttpURLConnection conn;
        DataOutputStream dos;
        String lineEnd = "\r\n";
        String twoHyphens = "--";
        String boundary = "------fileUpload";
        int bytesRead, bytesAvailable, bufferSize;
        byte[] buffer;
        int maxBufferSize = 1024 * 1024;
        File sourceFile = new File(filePath);


        try {
            FileInputStream fileInputStream = new FileInputStream(sourceFile);
            Log.v("FileUploadUrl", uploadUrl);
            URL url = new URL(uploadUrl);
            Log.v("FileUploadUrl", url.toString());
            // Open a HTTP  connection to  the URL
            conn = (HttpURLConnection) url.openConnection();
            conn.setConnectTimeout(1800000);
            conn.setChunkedStreamingMode(0);
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use a Cached Copy
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("ENCTYPE", "multipart/form-data");
            conn.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + boundary);
            if (syncType != null && !syncType.equalsIgnoreCase("F")) {
                conn.setRequestProperty("filename", "");
            } else {
                conn.setRequestProperty("filename", filePath);
            }
            conn.setRequestProperty("token", token);
            conn.setRequestProperty("user_id", userID);
            conn.setRequestProperty("upload_type", uploadType);

            String lastDate = databaseHelperExtended.getSyncDateAndTime();
            conn.setRequestProperty("first_sync", !lastDate.isEmpty()? "N":"Y");
            LogHelper.printLog("TAG_UPLOAD_FIRST","ISFIRST_SYNC::: "+(!lastDate.isEmpty()? "N":"Y"));

            conn.setRequestProperty(Const.WEB_SERVICES_PARAM.KEY_IMEI, imeiNumber);

            dos = new DataOutputStream(conn.getOutputStream());
            uploadFileSize = getFileSize(sourceFile);
            dos.writeBytes(twoHyphens + boundary + lineEnd);
            dos.writeBytes("Content-Disposition: form-data; name=\"file\";filename=\"" + filePath + "\"" + lineEnd);
            dos.writeBytes(lineEnd);
            // create a buffer of  maximum size
            bytesAvailable = fileInputStream.available();
            bufferSize = Math.min(bytesAvailable, maxBufferSize);
            buffer = new byte[bufferSize];
            // read file and write it into form...
            bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            while (bytesRead > 0) {
                dos.write(buffer, 0, bufferSize);
                bytesAvailable = fileInputStream.available();
                bufferSize = Math.min(bytesAvailable, maxBufferSize);
                bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            }
            // send multipart form data necesssary after file data...
            dos.writeBytes(lineEnd);
            dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);
            // Responses from the server (code and message)
            int serverResponseCode = conn.getResponseCode();
            String serverResponseMessage = conn.getResponseMessage();
            Log.i("RESPONSE_TAG", "HTTP Response is : " + serverResponseMessage + ": " + serverResponseCode);

            try {
                BufferedReader inStream = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                DataInputStream x = new DataInputStream(conn.getInputStream());
                String str="";

                while ((str = inStream.readLine()) != null) {
                    Log.e("FileUploadSOFRESPONSE", "SOF Server Response" + str);
                    LogHelper.printLog("TIME_forSync", "FileUploadStopTimer" + System.currentTimeMillis());
                    if (!str.trim().isEmpty()){
                        JSONObject jsonObject = new JSONObject(str);
                        String message = jsonObject.getString("message");
                        if (jsonObject.optBoolean("status")) {

                            String date_folder = jsonObject.getString("date_folder");
                            syncLogId = jsonObject.getString("sync_log_id");
                            LogHelper.printLog("SYNC_LOG_ID_SYNC", "SYNC LOG ID::: " + syncLogId);
//                            Content.setPreferenceValue(context, Const.dateUpload, date_folder);
                            SynchronizationService.dateFolder = date_folder;
                            serverResponseCode = 200;

                        } else {
                            serverResponseCode = 100;
                            error_code = jsonObject.optString("msg_code");
                            exceptionDetails = message;
                        }
                    }
                }
                inStream.close();
            } catch (IOException ioex) {
                exceptionDetails = AppUtil.getStackTrace(ioex);
                callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
                databaseHelperExtended.insertTransactionErrorLog(context, this.getClass().getSimpleName(), "uploadFile", exceptionDetails, exceptionDetails, "",syncLogId,userID);
                Log.e("FileUploadSOFERROR", "SOF error: " + ioex.getMessage(), ioex);
            }

            fileInputStream.close();
            dos.flush();
            dos.close();

//            Check if Response is SuccessFull Or Nor
            if (serverResponseCode == 200) {
                long FUpload_End = System.currentTimeMillis();
                Log.e("TAG_SYNC_FILE_UPLOAD", "SYNC FILE UPLOADED SUCCESSFULLY");
                try {
                    long difference = AppUtil.getMillisDifference(FUpload_Start, FUpload_End);
                    Log.v("SYNCTRACK", "UFS_ TIME::" + difference);

                    Uri uri = Uri.fromFile(new File(filePath));
                    callback.onFileUploadSuccess(uri.getLastPathSegment(), syncLogId, String.valueOf(difference), uploadFileSize,uploadType);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {

                Log.e("TAG_SYNC_FILE_UPLOAD", "SYNC FILE UPLOADED FAILED");
                if (error_code.equalsIgnoreCase(SyncErrorCode.ERROR_501)) {
                    callback.onFileUploadFail(Integer.parseInt(SyncErrorCode.ERROR_501), exceptionDetails, uploadFileSize,uploadType);
                } else {
                    exceptionDetails = exceptionDetails + " File Upload Server Error:" + serverResponseCode;
                    callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
                }

            }
        } catch (MalformedURLException ex) {
            exceptionDetails = AppUtil.getStackTrace(ex);
            callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
            ex.printStackTrace();
        } catch (Exception e) {
            exceptionDetails = AppUtil.getStackTrace(e);
            callback.onFileUploadFail(SyncErrorCode.ERROR_104, exceptionDetails, uploadFileSize,uploadType);
            e.printStackTrace();
        }
    }

}
